"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const report_1 = require("stryker-api/report");
const fs = require("mz/fs");
const chalk = require("chalk");
const _ = require("lodash");
const tempy = require('tempy');
const path = require("path");
class default_1 {
    constructor(_, baseline = read(), write = console.log) {
        this.baseline = baseline;
        this.write = write;
        this.filename = path.join(tempy.directory(), 'stryker.baseline.js');
    }
    onAllMutantsTested(results) {
        this.surviving = results.filter(_ => _.status === report_1.MutantStatus.Survived || _.status === report_1.MutantStatus.NoCoverage);
        if (!this.baseline) {
            this.ReportWithNoBaselineFile(this.surviving);
        }
        else {
            this.ReportDifferenceWithBaseline(this.surviving, this.baseline);
        }
    }
    wrapUp() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.write(`Saved a new baseline file: ${this.filename}`);
            yield fs.writeFile(this.filename, JSON.stringify(this.surviving));
        });
    }
    compare(lhs, rhs) {
        return lhs.mutatedLines === rhs.mutatedLines
            && lhs.originalLines === rhs.originalLines
            && lhs.mutatorName === rhs.mutatorName
            && lhs.sourceFilePath === rhs.sourceFilePath;
    }
    ReportWithNoBaselineFile(surviving) {
        if (surviving.length > 0) {
            this.write(chalk.yellow(chalk.bold('No hard feelings.'), 'Mutants survived but it may be not your fault after all.'));
            this.write('Copy the baseline file mentioned at the end of this log to the root dir of your project.');
            this.write('Mind though that including a baseline file at this point will let them live on forever...');
        }
        else {
            this.write(chalk.yellow(chalk.green('You rock!'), 'No surviving mutants at all.'));
            this.write('Who needs a baseline file anyway?!');
        }
    }
    ReportDifferenceWithBaseline(surviving, baseline) {
        let fresh = _.differenceWith(surviving, this.baseline, (lhs, rhs) => this.compare(lhs, rhs));
        if (fresh.length === 0) {
            this.ReportNoNewSurvivingMutants(surviving, baseline);
        }
        else {
            this.ReportNewSurvivingMutants(surviving, baseline);
        }
    }
    ReportNoNewSurvivingMutants(surviving, baseline) {
        if (surviving.length === baseline.length) {
            this.write(chalk.green(chalk.bold('Good job!'), 'At least you didn\'t create new surviving mutants.'));
        }
        else {
            this.write(chalk.green(chalk.bold('Great job!'), 'You killed some mutants that survived before.'));
            this.write('You should update the baseline file right now!');
            this.write('Overwrite your baseline file with the one mentioned at the end of this log.');
        }
    }
    ReportNewSurvivingMutants(surviving, baseline) {
        let gone = _.differenceWith(baseline, surviving, (lhs, rhs) => this.compare(lhs, rhs));
        if (gone.length === 0) {
            this.write(chalk.red(chalk.bold('Shame on you!'), 'New mutants survived. Let\'s go kill them!'));
            this.write('If you can\'t kill them you should update the baseline file.');
        }
        else {
            this.write(chalk.yellow(chalk.bold('Mixed feelings.'), 'You killed some mutants but also created some new surviving.'));
            this.write('Mind that updating the baseline file now will let them live on forever...');
        }
        this.write('Overwrite your baseline file with the one mentioned at the end of this log.');
    }
}
exports.default = default_1;
function read(filename = 'stryker.baseline.js') {
    return fs.existsSync(filename) ? JSON.parse(fs.readFileSync(filename, 'utf-8')) : undefined;
}
exports.read = read;
//# sourceMappingURL=BaselineReporter.js.map